#!/usr/bin/python

import numpy as np
from numpy import linalg
import math, sys, os, gc , re , time
import pandas as pd
import argparse

parser=argparse.ArgumentParser(add_help=False,formatter_class=argparse.RawDescriptionHelpFormatter,description='Assessment of Predicted Distances.',epilog="""An example:\npython apd.py -n example/seq.pdb -p example/seq.dist \n """)

arghelp=parser.add_argument_group('help information')
arghelp.add_argument("-h", "--help",action="help",help="show this message and exit")

argrequired=parser.add_argument_group('mandatory arguments')
argrequired.add_argument("-n",dest='PDB', required=True, type=str,help="native structure in PDB format")
argrequired.add_argument("-p",dest='PRE', required=True, type=str, help="distance prediction in CASP format")


argoptional=parser.add_argument_group('optional arguments')
argoptional.add_argument('-m', type=str, dest='MODE', default=0, choices=[1,2,3,0], help='(default 0)  1:predition-oriented only; 2:native-oriented only; 3:full-list only; 0: perform all assessment (1+2+3)')
argoptional.add_argument("-c",type=int,dest="CSV",default=0,choices=[0,1],help="(default 0)  0: results to csv file; 1: printed on screen")


args=parser.parse_args()

pdbfile=args.PDB
predfile=args.PRE
mode=args.MODE
csv=args.CSV

separation=12
topn=15
topnc=1

DICT = {
	'GLY':'G',
	'ALA':'A',
	'VAL':'V',
	'LEU':'L',
	'ILE':'I',
	'SER':'S',
	'THR':'T',
	'CYS':'C',
	'MET':'M',
	'PRO':'P',
	'ASP':'D',
	'ASN':'N',
	'GLU':'E',
	'GLN':'Q',
	'LYS':'K',
	'ARG':'R',
	'HIS':'H',
	'PHE':'F',
	'TYR':'Y',
	'TRP':'W',
	'MSE':'M',
	'ASX':'N',
	'GLX':'Q',
	'UNK':'G',
	'HSD':'H'}

def get_code(alpha):
	code = DICT.get(alpha)
	if code is None:
		print ( "[error]miss %s in the dictionary" % alpha )
		system.exit(1)
	return code

def convert2number(coord_str):
	lens = len(coord_str)
	coord = np.zeros( (lens) )
	try:
		for i in range(lens):
			coord[i] = float(coord_str[i])
			if math.isinf(coord[i]) or math.isnan(coord[i]):
				return (1, coord)
		return (0, coord)
	except Exception:
		print ( "[error] cast a string to a float number" )
		return (1, coord)

def get_groups(filename):
	groups = []
	try:
		fo = open(filename, "r",10240)
		try:
			cur_res_no = ""
			las_res_no = ""
			atomname = ""
			for line in fo:
				if line.startswith('ATOM') :
					atomname = line[12:16]
					if atomname.find('CA') != -1 or atomname.find('CB') != -1:
						cur_res_no = line[22:26]
						if cur_res_no == las_res_no:
							group = groups[len(groups)-1]
							group.append(line)
						else:
							group = [line]
							groups.append(group)
						las_res_no = cur_res_no
					continue
				if line.startswith('TER'):
					break
			return groups
		finally:
			fo.close()
	except Exception:
		print ( "[error] in file %s" % (filename) )
		sys.exit(1)

def get_fastas_coords(filename):
	name = os.path.basename(filename).split('.')[0]
	groups = get_groups(filename)
	lens = len(groups)
	coords = np.zeros((lens,3), np.float32)
	positions = np.zeros((lens), np.int16)
	fastas = ['>gi|'+name+'|etc' + os.linesep]
	fasta = ""
	max_lens = 100000
	
	for g in range(lens):
		group = groups[g]
		line = group[0]
		for i in range(1,len(group)):
			atomname = group[i][12:16]
			if atomname.find('CB') != -1 :
				line = group[i]
				break
		
		coord_str = [line[30:38],line[38:46],line[46:54]]
		(sig,coord) = convert2number(coord_str)
		if sig != 0:
			print ( "[error] invalide number found in line [%s] in file %s" % (line, filename) )
			sys.exit(1)
		#add the positions of residues
		positions[g] = line[22:26]
		if g>1 and positions[g] <= positions[g-1]:
			print ( "[error] invalide renumber found in line [%s] in file %s" % (line, filename) )
			sys.exit(1)
		coords[g] = coord
		fasta=fasta+get_code(line[17:20])
		if len(fasta) == max_lens :
			fastas.append(fasta + os.linesep)
			fasta = ""

	fastas.append(fasta + os.linesep)
	del groups
	gc.collect()

	return (fastas, coords, positions)

def get_distance_matrix(coords):
	lens = len(coords)
	distances = np.zeros((lens, lens), np.float32)
	for i in range(lens):
		for j in range(i):
			distances[i][j] = distances[j][i] = linalg.norm(coords[i]-coords[j])
	return distances

def get_distance_distribution(predfile,ress,separation):
	#renumber according to native pdb
	map={}
	allpairs=[]
	for i in range(len(ress)):
		map[ress[i]]=i+1
	
	for i in ress:
		for j in ress:
			if j<i and abs(i-j)>=separation:
				allpairs.append((map[i]-1,map[j]-1))

	prematrix=np.zeros((len(ress),len(ress),10),dtype=float)
			
	
	idx=[]
	idxc=[]

	#predis=np.zeros((len(ress),len(ress),9),dtype=float)
	with open(predfile,'r') as f:
		for line in f:
			line=line.strip().split()
			if line==[]:
				continue
			if not line[0].isdigit():
				continue
			else:
				try:
					i=int(line[0])
					j=int(line[1])
					if i in map.keys() and j in map.keys():
						#check predicted distance distribution
						if abs(float(line[3])+float(line[4])+float(line[5])-float(line[2]))>0.005:
							print("[error] residue pair (%s,%s) p1+p2+p3 does not equal to p0 (tolerance: 0.005)!"%(str(i),str(j)))
							sys.exit(1)
						total=0
						for k in range(10):
							total+=float(line[k+3])
						if abs(total-1)>0.005:
							if total>1:
								print("[error] residue pair (%s,%s) p1+..+p10 does not equal to 1 (tolerance: 0.005)!"%(str(i),str(j)))
								sys.exit(1)
							else:
								print("[warning] residue pair (%s,%s) p1+..+p10=%s does not equal to 1 (tolerance: 0.005)!"%(str(i),str(j),str(total)))
						#read predicted distance distribution
						if abs(i-j) >= separation:
						#if abs(map[i]-map[j])>=separation:
							idx.append([map[i]-1,map[j]-1,float(line[3]),float(line[4]),float(line[5]),float(line[6]),float(line[7]),float(line[8]),float(line[9]),float(line[10]),float(line[11]),total-float(line[12])])
							idxc.append([map[i]-1,map[j]-1,float(line[2])])
							for k in range(10):
								prematrix[map[i]-1][map[j]-1][k]=prematrix[map[j]-1][map[i]-1][k]=float(line[3+k])

						# for k in range(9):
							# predis[map[i]-1][map[j]-1][k]=predis[map[j]-1][map[i]-1]=float(line[k+3])
				except Exception as e:
					print(e)
					print("[error] your input file is not in standard CASP format!")
					sys.exit(1)
	return np.array(idx),np.array(idxc),prematrix,map,allpairs

def cp(precontop,realdis,total):
	TP=0
	for pred in precontop:
		i=int(pred[0])
		j=int(pred[1])
		if realdis[i][j]<=8 and pred[-1] != 0:
			TP+=1
	return round(TP/total,3)


def dp(predis,realdis,total):
	TP=0
	for pred in predis:
		i=int(pred[0])
		j=int(pred[1])
		predistri=pred[2:]
		predist=weightedis(predistri)
		if abs(predist-realdis[i][j])<=2 and realdis[i][j]<=20 :
			TP+=predistri[-1]
	return round(TP/total,3)

def fc(predis,realdis,total):
	w=0.5
	cer=0
	for pred in predis:
		i=int(pred[0])
		j=int(pred[1])
		predistri=pred[2:-1]
		bin=realbin(realdis[i][j])
		if bin<9:
			if bin==0:
				cer+=predistri[bin]+w*predistri[bin+1]
			elif bin==8:
				cer+=predistri[bin]+w*predistri[bin-1]
			else:
				cer+=predistri[bin]+w*predistri[bin+1]+w*predistri[bin-1]
	return round(cer/total,3)
			
def error(predistop,realdis,total):
	aerr=0
	rerr=0
	for pred in predistop:
		i=int(pred[0])
		j=int(pred[1])
		predistri=pred[2:]
		predist=weightedis(predistri)
		aerr+=abs(predist-realdis[i][j])
		rerr+=abs(predist-realdis[i][j])/realdis[i][j]
	return round(aerr/total,3),round(rerr/total,3)

def corr(predistop,realdis):
	preds=[]
	reals=[]
	for pred in predistop:
		i=int(pred[0])
		j=int(pred[1])
		predistri=pred[2:]
		predist=weightedis(predistri)
		preds.append(predist)
		reals.append(realdis[i][j])
	if len(predistop) > 1:
		return round(np.corrcoef(preds,reals)[0][1],3)
	else:
		return "NaN"

def mf(predis,realdis,realcounts,prebin):
	tp=[0]*9
	pre=[0]*9
	for pred in predis:
		i=int(pred[0])
		j=int(pred[1])
		predistri=pred[2:]
		preb=prebin(predistri)
		realb=realbin(realdis[i][j])
		if preb==realb and realb<9:
			tp[preb]+=1
		elif abs(preb-realb)==1 and realb<9 and preb<9:
			tp[preb]+=0.5
		if preb<9:
			pre[preb]+=1
		
	precisions=[0]*9
	recalls=[0]*9
	f1=0
	non0bins=0
	for i in range(9):
		if pre[i] != 0:
			precisions[i]=tp[i]/pre[i]
		if realcounts[i] != 0:
			non0bins+=1
			recalls[i]=tp[i]/realcounts[i]
		if precisions[i]*recalls[i] != 0:
			f1+=2*precisions[i]*recalls[i]/(precisions[i]+recalls[i])
	

	precision=np.sum(precisions)/non0bins
	recall=np.sum(recalls)/non0bins
	f1/=non0bins
	
	return round(precision,3),round(recall,3),round(f1,3)		

def DLDDT(prematrix,realdis,separation,map):
	mapre={value: key for key, value in map.items()}

	isum=0
	for i in range(len(prematrix)):
		jcount=0
		jpsum1=0
		jpsum2=0
		jpsum3=0
		jpsum4=0
		for j in range(len(prematrix)):
			if abs(mapre[i+1]-mapre[j+1])>=separation and realdis[i][j]<=20:
				jcount+=1
				diff=abs(realdis[i][j]-weightedis(prematrix[i][j]))
				
				if diff<=0.5:
					jpsum1+=np.sum(prematrix[i][j][:-1])
					jpsum2+=np.sum(prematrix[i][j][:-1])
					jpsum3+=np.sum(prematrix[i][j][:-1])
					jpsum4+=np.sum(prematrix[i][j][:-1])
					
				elif diff<=1:
					jpsum2+=np.sum(prematrix[i][j][:-1])
					jpsum3+=np.sum(prematrix[i][j][:-1])
					jpsum4+=np.sum(prematrix[i][j][:-1])
					
				elif diff<=2:
					jpsum3+=np.sum(prematrix[i][j][:-1])
					jpsum4+=np.sum(prematrix[i][j][:-1])
						
				elif diff<=4:
					jpsum4+=np.sum(prematrix[i][j][:-1])

		if jcount != 0:
			isum+=(jpsum1+jpsum2+jpsum3+jpsum4)/jcount

	return round(isum/(4*len(prematrix)),3)

def mfc(allpairs,prematrix,realdis):
	fc=0
	w=0.5
	realbin_arr=[0]*10
	cer_arr=[0]*10
	for (i,j) in allpairs:
		bin=realbin(realdis[i][j])
		realbin_arr[bin]+=1
		if bin==0:
			cer_arr[bin]+=prematrix[i][j][bin]+w*prematrix[i][j][bin+1]
		elif bin==8:
			cer_arr[bin]+=prematrix[i][j][bin]+w*prematrix[i][j][bin-1]
		elif bin==9:
			cer_arr[bin]+=prematrix[i][j][bin]
		else:
			cer_arr[bin]+=prematrix[i][j][bin]+w*(prematrix[i][j][bin-1]+prematrix[i][j][bin+1])
	non0bins=0
	for s in range(10):
		if realbin_arr[s] != 0:
			fc+=cer_arr[s]/realbin_arr[s]
			non0bins+=1
	return  round(fc/non0bins,3)

def mf_full(allpairs,prematrix,realdis,realcounts,prebin):
	tp=[0]*10
	pre=[0]*10
	for (i,j) in allpairs:
		predistri=prematrix[i][j]
		preb=prebin(predistri)
		realb=realbin(realdis[i][j])
		if preb==realb:
			tp[preb]+=1
		elif abs(preb-realb)==1 and realb<9 and preb<9:
			tp[preb]+=0.5

		pre[preb]+=1
		
	precisions=[0]*10
	recalls=[0]*10
	f1=0
	non0bins=0
	for i in range(10):
		if pre[i] != 0:
			precisions[i]=tp[i]/pre[i]
		if realcounts[i] != 0:
			recalls[i]=tp[i]/realcounts[i]
			non0bins+=1
		if precisions[i]*recalls[i] != 0:
			f1+=2*precisions[i]*recalls[i]/(precisions[i]+recalls[i])
	
	
	precision=np.sum(precisions)/non0bins
	recall=np.sum(recalls)/non0bins
	f1/=non0bins
	
	return round(precision,3),round(recall,3),round(f1,3)		


def weightedis(array):
	wdis=0 
	totalprob=np.sum(array[:-1])
	if totalprob==0:
		return 25
	else:
		for k in range(9):
			middis=3+k*2
			preprob=array[k]
			wdis+=middis*preprob
		
		wdis/=totalprob
		return wdis
		

def realbin(dist):
	if dist<=20 and dist>2:
		if (dist-2)%2==0:
			bin=int((dist-2)/2)-1
		else:
			bin=int((dist-2)/2)
	elif dist>20:
		bin=9
	else:
		bin=0
	return bin

def prebin_predicted(predistri):
	bin=np.argmax(predistri[:-1])
	return bin

def prebin_full(predistri):
	if np.sum(predistri[:-1])>=0.5:
		bin=np.argmax(predistri[:-1])
	else:
		bin=9
	return bin
	

def realcount(realdis,separation,map):
	mapre={value: key for key, value in map.items()}
	realcounts=[0]*10
	realpairs=[]
	for j in range(len(realdis)):
		for i in range(j):
			if abs(mapre[i+1]-mapre[j+1])>= separation:
				realcounts[realbin(realdis[i][j])]+=1
				if realdis[i][j]<=20:
					realpairs.append((i,j))
	return realcounts,realpairs


def prediction_eva(allpairs,realcounts,idx,idxc,topn,topnc,ress,realdis,csv):
	
	if len(allpairs)==0:
		contact_precision="NaN"
		distance_precision_predicted="NaN"
		fuzzy_certainty_predicted="NaN"
		absolute_error="NaN"
		relative_error="NaN"
		correlation="NaN"
		macro_fuzzy_precision_predicted="NaN"
		macro_fuzzy_recall_predicted="NaN"
		macro_fuzzy_f1_predicted="NaN"
	
	else:
	
		if np.sum(realcounts[:-1])==0:
			contact_precision="NaN"
			distance_precision_predicted="NaN"
			fuzzy_certainty_predicted="NaN"
			absolute_error="NaN"
			relative_error="NaN"
			correlation="NaN"
			macro_fuzzy_precision_predicted="NaN"
			macro_fuzzy_recall_predicted="NaN"
			macro_fuzzy_f1_predicted="NaN"


		elif len(idx)==0:
			distance_precision_predicted=0
			fuzzy_certainty_predicted=0
			absolute_error="NaN"
			relative_error="NaN"
			correlation="NaN"
			macro_fuzzy_precision_predicted=0
			macro_fuzzy_recall_predicted=0
			macro_fuzzy_f1_predicted=0
			
			if np.sum(realcounts[:3])==0:
				contact_precision="NaN"
			elif len(idxc)==0:
				contact_precision=0
			else:
				precon = idxc[np.flip(np.argsort(idxc[:,2]),axis=0)]
				topnLc=min(int(topnc*len(ress)),len(precon))
				precontop=precon[:topnLc,:]
				contact_precision=cp(precontop,realdis,int(topnc*len(ress)))


				
		
		else:
		#prediction-oriented
			if np.sum(realcounts[:3])==0:
				contact_precision="NaN"
			elif len(idxc)==0:
				contact_precision=0
			else:
				precon = idxc[np.flip(np.argsort(idxc[:,2]),axis=0)]
				topnLc=min(int(topnc*len(ress)),len(precon))
				precontop=precon[:topnLc,:]
				contact_precision=cp(precontop,realdis,int(topnc*len(ress)))

			predis=idx[np.flip(np.argsort(idx[:,-1]),axis=0)]		
			topnL=min(int(topn*len(ress)),len(predis))
			predistop=predis[:topnL,:]
			nonzero=[i for i in range(topnL) if predistop[i,-1]!=0 and np.sum(predistop[i][:-1]!=0)]


			if len(nonzero)==0:
				distance_precision_predicted=0
				fuzzy_certainty_predicted=0
				absolute_error="NaN"
				relative_error="NaN"
				correlation="NaN"
				macro_fuzzy_precision_predicted=0
				macro_fuzzy_recall_predicted=0
				macro_fuzzy_f1_predicted=0			
			else:
				predistop=predistop[nonzero]
				distance_precision_predicted=dp(predistop,realdis,int(topn*len(ress)))
				fuzzy_certainty_predicted=fc(predistop,realdis,int(topn*len(ress)))
				absolute_error,relative_error=error(predistop,realdis,len(predistop))
				correlation=corr(predistop,realdis)				
				macro_fuzzy_precision_predicted,macro_fuzzy_recall_predicted,macro_fuzzy_f1_predicted=mf(predistop,realdis,realcounts,prebin_predicted)
					
	if csv==0:
		df=pd.DataFrame({"Contact precision":contact_precision,"Distance precision":distance_precision_predicted,"Fuzzy certainty":fuzzy_certainty_predicted,"Absolute error":absolute_error,"Relative error":relative_error,"Pearson's correlation coefficient":correlation,"Macro fuzzy precision":macro_fuzzy_precision_predicted,"Macro fuzzy recall":macro_fuzzy_recall_predicted,"Macro fuzzy f1":macro_fuzzy_f1_predicted},index=['Value'])
		df=df.T
		df.to_csv("prediction_oriented.csv")
		
	else:
		print()
		print("=====predicted-oriented results=====")
		print("contact precision:",contact_precision)
		print("distance precision:",distance_precision_predicted)
		print("fuzzy certainty:",fuzzy_certainty_predicted)
		print("absolute error:",absolute_error)
		print("relative error:",relative_error)
		print("pearson's correlation coefficient:",correlation)
		print("macro fuzzy precision:",macro_fuzzy_precision_predicted)
		print("macro fuzzy recall:",macro_fuzzy_recall_predicted)
		print("macro fuzzy f1:",macro_fuzzy_f1_predicted)		
		print("====================================")
		print()
			
def native_eva(allpairs,realcounts,idx,realpairs,prematrix,realdis,map,csv):
	if len(allpairs)==0:
		
		dlddt="NaN"
		distance_precision_native="NaN"
		fuzzy_certainty_native="NaN"
		macro_fuzzy_precision_native="NaN"
		macro_fuzzy_recall_native="NaN"
		macro_fuzzy_f1_native="NaN"

	
	else:
	
		if np.sum(realcounts[:-1])==0:
			dlddt="NaN"
			distance_precision_native="NaN"
			fuzzy_certainty_native="NaN"
			macro_fuzzy_precision_native="NaN"
			macro_fuzzy_recall_native="NaN"
			macro_fuzzy_f1_native="NaN"

		elif len(idx)==0:

			dlddt=0
			distance_precision_native=0
			fuzzy_certainty_native=0
			macro_fuzzy_precision_native=0
			macro_fuzzy_recall_native=0
			macro_fuzzy_f1_native=0

				
		
		else:

			predis=idx[np.flip(np.argsort(idx[:,-1]),axis=0)]		
			predisnative=[]
			for k,(i,j) in enumerate(zip(predis[:,0],predis[:,1])):
				if (i,j) in realpairs:
					predisnative.append(predis[k])
					realpairs.remove((i,j))
				elif (j,i) in realpairs:
					predisnative.append(predis[k])
					realpairs.remove((j,i))	
			for (i,j) in realpairs:
				x=[i,j]+[0]*10
				predisnative.append(x)

			dlddt=DLDDT(prematrix,realdis,separation,map)
			distance_precision_native=dp(predisnative,realdis,np.sum(realcounts[:-1]))
			fuzzy_certainty_native=fc(predisnative,realdis,np.sum(realcounts[:-1]))
			macro_fuzzy_precision_native,macro_fuzzy_recall_native,macro_fuzzy_f1_native=mf(predisnative,realdis,realcounts,prebin_full)

	
	if csv==0:
		df=pd.DataFrame({"Distogram LDDT":dlddt,"Distance precision":distance_precision_native,"Fuzzy certainty":fuzzy_certainty_native,"Macro fuzzy precision":macro_fuzzy_precision_native,"Macro fuzzy recall":macro_fuzzy_recall_native,"Macro fuzzy f1":macro_fuzzy_f1_native},index=['Value'])
		df=df.T
		df.to_csv("native_oriented.csv")
	else:
		print()
		print("=====native-oriented results=====")
		print("distogram LDDT:",dlddt)	
		print("distance precision:",distance_precision_native)
		print("fuzzy certainty:",fuzzy_certainty_native)
		print("macro fuzzy precision:",macro_fuzzy_precision_native)
		print("macro fuzzy recall:",macro_fuzzy_recall_native)
		print("macro fuzzy f1:",macro_fuzzy_f1_native)
		print("=================================")
		print()
		
	
def full_eva(allpairs,prematrix,realdis,realcounts,csv):
	if len(allpairs)==0:
		macro_fuzzy_certainty="NaN"
		macro_fuzzy_precision_full="NaN"
		macro_fuzzy_recall_full="NaN"
		macro_fuzzy_f1_full="NaN"
	
	else:

		#full-list
		macro_fuzzy_certainty=mfc(allpairs,prematrix,realdis)
		macro_fuzzy_precision_full,macro_fuzzy_recall_full,macro_fuzzy_f1_full=mf_full(allpairs,prematrix,realdis,realcounts,prebin_full)

	if csv==0:
		df=pd.DataFrame({"Macro fuzzy certainty":macro_fuzzy_certainty,"Macro fuzzy precision":macro_fuzzy_precision_full,"Macro fuzzy recall":macro_fuzzy_recall_full,"Macro fuzzy f1":macro_fuzzy_f1_full},index=["Value"])
		df=df.T
		df.to_csv("full_list.csv")
	else:
		print()
		print("=====full-list results=====")
		print("macro fuzzy certainty:",macro_fuzzy_certainty)
		print("macro fuzzy precision:",macro_fuzzy_precision_full)
		print("macro fuzzy recall:",macro_fuzzy_recall_full)
		print("macro fuzzy f1:",macro_fuzzy_f1_full)
		print("===========================")
		print()
		
		

def main():
	
	#read native pdb. COOR: coordinate of residues. RESS: residue numbers in pdb file
	print("reading input files...")
	fasta, coor, ress = get_fastas_coords(pdbfile)
	realdis=get_distance_matrix(coor)

		
	#read and renumber predicted distance in CASP format
	idx,idxc,prematrix,map,allpairs=get_distance_distribution(predfile,ress,separation)
	realcounts,realpairs=realcount(realdis,separation,map)
	
	if len(allpairs)==0:
		print("[warning] no residue pairs with separation>=12 in your input pdb!")
	elif np.sum(realcounts[:-1])==0:
		print("[warning] no residue pairs with distance<=20 and separation>=12 in your input pdb!")
	elif np.sum(realcounts[:3])==0:
		print("[warning] no residue pairs with distance<=8 and separation>=12 in your input pdb!")
	
	#perform assessment
	if mode==0:
		print("prediction-oriented assessment ...")
		prediction_eva(allpairs,realcounts,idx,idxc,topn,topnc,ress,realdis,csv)
		
		print("native-oriented assessment ...")
		native_eva(allpairs,realcounts,idx,realpairs,prematrix,realdis,map,csv)
		
		print("full-list assessment...")
		full_eva(allpairs,prematrix,realdis,realcounts,csv)
		
	elif mode==1:
		print("prediction-oriented assessment ...")
		prediction_eva(allpairs,realcounts,idx,idxc,topn,topnc,ress,realdis,csv)
	
	elif mode==2:
		print("native-oriented assessment ...")
		native_eva(allpairs,realcounts,idx,realpairs,prematrix,realdis,map,csv)
	
	elif mode==3:
		print("full-list assessment...")
		full_eva(allpairs,prematrix,realdis,realcounts,csv)
	
	
	print("done")
	

	

if __name__ == "__main__":
	main()

	
