#!/usr/bin/python
import numpy as np
import sys,os,getopt
import scipy
import scipy.spatial
import math
from collections import defaultdict
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt





DICT = {
	'GLY':'G',
	'ALA':'A',
	'VAL':'V',
	'LEU':'L',
	'ILE':'I',
	'SER':'S',
	'THR':'T',
	'CYS':'C',
	'MET':'M',
	'PRO':'P',
	'ASP':'D',
	'ASN':'N',
	'GLU':'E',
	'GLN':'Q',
	'LYS':'K',
	'ARG':'R',
	'HIS':'H',
	'PHE':'F',
	'TYR':'Y',
	'TRP':'W',
	'MSE':'M',
	'ASX':'N',
	'GLX':'Q',
	'UNK':'G',
	'HSD':'H'}



def parse_pdb(pdbfile,chain_number):
    dict_pdb=defaultdict(list)
    fr=open(pdbfile)
    lines=fr.readlines()
    for line in lines:
        if line.startswith('ATOM'):
            chain=line[21]
            dict_pdb[chain].append(line)
    chains=list(dict_pdb.keys())
    key=chains[chain_number-1]
    return (dict_pdb[key])

def get_xyz(pdb):
    target=['N','CA','CB','C']
    dict_xyz_all=defaultdict(dict)
    for line in pdb:
        residue = line[17:20].strip()
        ####remove nonstandard amino acid
        if DICT.get(residue):
            atomname = line[12:16].strip()
            if atomname in target:
                res_no=line[22:26].strip()
                coords_str=[line[30:38], line[38:46], line[46:54]]
                coords=[float(k) for k in coords_str]
                dict_xyz_all[res_no][atomname]=coords
    essential=['N','CA','C']
    dict_xyz={}
    for key in dict_xyz_all:
        item=dict_xyz_all[key]
        atom_set=item.keys()
        if set(essential).issubset(set(atom_set)):
            dict_xyz[key]=item


    return (dict_xyz)


def get_info(xyz,dmax):
    nres= len(xyz)
    position=list(xyz.keys())
    N = np.stack([xyz[position[i]]['N'] for i in range(nres)])
    CA = np.stack([xyz[position[i]]['CA'] for i in range(nres)])
    C= np.stack([xyz[position[i]]['C'] for i in range(nres)])
    # recreate Cb given N,Ca,C
    b = CA - N
    c = C - CA
    a = np.cross(b, c)
    CB = -0.58273431 * a + 0.56802827 * b - 0.54067466 * c + CA
    ###replace CB with real xyz
    for i in range(nres):
        value=xyz[position[i]]
        if 'CB' in value.keys():
             CB[i]=xyz[position[i]]['CB']

    # fast neighbors search
    kdCb = scipy.spatial.cKDTree(CB)
    indices = kdCb.query_ball_tree(kdCb, dmax)
    # indices of contacting residues
    idx = np.array([[i, j] for i in range(len(indices)) for j in indices[i] if i != j]).T
    idx0 = idx[0]
    idx1 = idx[1]

    # Cb-Cb distance matrix
    dist6d = np.zeros((nres, nres))
    dist6d[idx0, idx1] = np.linalg.norm(CB[idx1] - CB[idx0], axis=-1)
    ##Cb-Cb distance matrix for plot
    dist6d_plot = np.eye(nres)*(-21)+25
    dist6d_plot[idx0, idx1] = np.linalg.norm(CB[idx1] - CB[idx0], axis=-1)
	
    
    return (dist6d, dist6d_plot)
    

def plot(name,img):
    plt.figure()
    color_map = plt.imshow(img.astype(float))
    color_map.set_cmap("hot_r")
    plt.colorbar()
    plt.savefig(name, bbox_inches='tight', dpi=600, transparent=False)



def output(mtx):
    ###plot distance map
    name_dist='seq.errdist.png'
    #matplotlib.rcParams['image.cmap'] = 'gist_stern_r'
	
    plot(name_dist,mtx)


def usage():
    print(" usage: python3 visualize_err.py [options]")
    print(" options:")
    print(" -f native structure file in PDB format (mandatory)")
    print(" -p predicted distance file in CASP format (mandatory)")
    print(" -c probability cutoff to determine if the prediction falls into [0,20A], default:0.5.(i.e., for residue pair (i,j),  p(d(i,j)<20A)<c means the prediction for d(i,j) is > 20A.)")
    print(" -n chain number (optional), default: 1 (i.e., the first chain in pdb file)")
    print()



def weighted_avg_and_std(values, weights):
    if(len(values)==1):return values[0],0

    average = np.average(values, weights=weights)
    variance=np.array(values).std()
    return average, variance

def re_pre(prefile,pdbfile):
	#read and renumber the predicted distance matrix
	ress=[]
	with open(pdbfile,'r') as f:
		for line in f:
			line=line.strip()
			if line[0:4] == "ATOM":
				resnum=line[22:26].strip()
				if int(resnum) not in ress:
					ress.append(int(resnum))
		
	map={}

	for i in range(len(ress)):
		map[ress[i]]=i

	prematrix=np.zeros((len(ress),len(ress),10),dtype=float)
			


	#predis=np.zeros((len(ress),len(ress),9),dtype=float)
	with open(prefile,'r') as f:
		for line in f:
			line=line.strip().split()
			if line==[]:
				continue
			if not line[0].isdigit():
				continue
			else:
				try:
					i=int(line[0])
					j=int(line[1])
					if i in map.keys() and j in map.keys():
							for k in range(10):
								prematrix[map[i]][map[j]][k]=prematrix[map[j]][map[i]][k]=float(line[3+k])

				except Exception as e:
					print(e)
					print("[error] your input file is not in standard CASP format!")
					sys.exit(1)
	return prematrix


def dist(dat,pcut):
    nres=int(dat.shape[0])

    #pcut=0.05
    bin_p=0.01

    mat=np.zeros((nres, nres))

    
    for i in range(0, nres):
        for j in range(0, nres):
            
            if(j == i):
                mat[i][i]=4
                continue

            if(j<i):
                mat[i][j]=mat[j][i]
                continue

                       
            #check probability
            Praw = dat[i][j]
            first_bin=1
            first_d=5
            weight=0


            
            for P in Praw[first_bin:-1]:
                if(P>bin_p): weight += P
                
                
            if(weight < pcut):
                mat[i][j]=25
                continue


            Pnorm = [P for P in Praw[first_bin:-1]]

            probs=[]
            dists=[]
            xs = []
            ys = []
            dmax=0
            pmax=-1

            for k,P in enumerate(Pnorm):
                d = first_d + k*2
                if(P>pmax):
                    dmax=d
                    pmax=P
                #endif

                if(P>bin_p):
                    probs.append(P)
                    dists.append(d)
                #endif
                xs.append(d)

            e_dis=8;
            e_std=0;
            if(len(probs)==0):
                e_dis=dmax
                e_std=0;
            else:
                probs = [P/sum(probs) for P in probs]

                e_dis, e_std=weighted_avg_and_std(dists, probs)

            mat[i][j]=e_dis


    return mat


def main():
    opts, args = getopt.getopt(sys.argv[1:], "f:p:c:n:h")
    pdbfile='-1'
    prefile='-1'
    pcut=0.5  ##default probability cutoff to determine if the prediction falls into [0,20A]
    chain_number = 1  ##default the first chain
    for o, a in opts:
        if o == "-f":
            pdbfile = a
        elif o == "-p":
            prefile = a
        elif o == "-c":
            pcut=float(a)
        elif o == "-n":
            chain_number=int(a)
        elif o == '-h':
            usage()
            sys.exit(0)

    if (pdbfile == '-1' or prefile=='-1'):
        print('\n This script visualize the error map with inputs of a native structure file in PDB format and a predicted distance file in CASP format.\n')
        usage()
        exit(1)


    # native distance matrix
    pdb=parse_pdb(pdbfile,chain_number)
    xyz=get_xyz(pdb)
    (dist6d, nat_dismtx)=get_info(xyz,20)
	
    # predicted distance matrix
    dat=re_pre(prefile,pdbfile)
    pre_dismtx=dist(dat,pcut)
   	
    diffmtx=abs(nat_dismtx-pre_dismtx)
    output(diffmtx)
    print("seq.errdist.png generated successfully.")   
    

if __name__ == "__main__":
    main()

