#!/usr/bin/python
import numpy as np
import sys,os,getopt
import scipy
import scipy.spatial
import math
from collections import defaultdict
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt






DICT = {
	'GLY':'G',
	'ALA':'A',
	'VAL':'V',
	'LEU':'L',
	'ILE':'I',
	'SER':'S',
	'THR':'T',
	'CYS':'C',
	'MET':'M',
	'PRO':'P',
	'ASP':'D',
	'ASN':'N',
	'GLU':'E',
	'GLN':'Q',
	'LYS':'K',
	'ARG':'R',
	'HIS':'H',
	'PHE':'F',
	'TYR':'Y',
	'TRP':'W',
	'MSE':'M',
	'ASX':'N',
	'GLX':'Q',
	'UNK':'G',
	'HSD':'H'}



def parse_pdb(pdbfile,chain_number):
    dict_pdb=defaultdict(list)
    fr=open(pdbfile)
    lines=fr.readlines()
    for line in lines:
        if line.startswith('ATOM'):
            chain=line[21]
            dict_pdb[chain].append(line)
    chains=list(dict_pdb.keys())
    key=chains[chain_number-1]
    return (dict_pdb[key])

def get_xyz(pdb):
    target=['N','CA','CB','C']
    dict_xyz_all=defaultdict(dict)
    for line in pdb:
        residue = line[17:20].strip()
        ####remove nonstandard amino acid
        if DICT.get(residue):
            atomname = line[12:16].strip()
            if atomname in target:
                res_no=line[22:26].strip()
                coords_str=[line[30:38], line[38:46], line[46:54]]
                coords=[float(k) for k in coords_str]
                dict_xyz_all[res_no][atomname]=coords
    essential=['N','CA','C']
    dict_xyz={}
    for key in dict_xyz_all:
        item=dict_xyz_all[key]
        atom_set=item.keys()
        if set(essential).issubset(set(atom_set)):
            dict_xyz[key]=item


    return (dict_xyz)


def get_info(xyz,dmax):
    nres= len(xyz)
    position=list(xyz.keys())
    N = np.stack([xyz[position[i]]['N'] for i in range(nres)])
    CA = np.stack([xyz[position[i]]['CA'] for i in range(nres)])
    C= np.stack([xyz[position[i]]['C'] for i in range(nres)])
    # recreate Cb given N,Ca,C
    b = CA - N
    c = C - CA
    a = np.cross(b, c)
    CB = -0.58273431 * a + 0.56802827 * b - 0.54067466 * c + CA
    ###replace CB with real xyz
    for i in range(nres):
        value=xyz[position[i]]
        if 'CB' in value.keys():
             CB[i]=xyz[position[i]]['CB']

    # fast neighbors search
    kdCb = scipy.spatial.cKDTree(CB)
    indices = kdCb.query_ball_tree(kdCb, dmax)
    # indices of contacting residues
    idx = np.array([[i, j] for i in range(len(indices)) for j in indices[i] if i != j]).T
    idx0 = idx[0]
    idx1 = idx[1]

    # Cb-Cb distance matrix
    dist6d = np.zeros((nres, nres))
    dist6d[idx0, idx1] = np.linalg.norm(CB[idx1] - CB[idx0], axis=-1)
    ##Cb-Cb distance matrix for plot
    dist6d_plot = np.eye(nres)*(-16)+20
    dist6d_plot[idx0, idx1] = np.linalg.norm(CB[idx1] - CB[idx0], axis=-1)

    
    return (dist6d, dist6d_plot)
    

def plot(name,img):
    plt.figure()
    color_map = plt.imshow(img.astype(float))
    color_map.set_cmap("hot")
    plt.colorbar()
    plt.savefig(name, bbox_inches='tight', dpi=600, transparent=False)



def output(xyz):
    (dist6d, dist6d_plot)=get_info(xyz,20)

    ###plot distance map
    name_dist='seq.natdist.png'
    plot(name_dist,dist6d_plot)


def usage():
    print(" usage: python3 visualize_nat.py [options]")
    print(" options:")
    print(" -f pdb file (mandatory)")
    print(" -n chain number (optional), default: 1 (i.e., the first chain in pdb file)")

def main():
    opts, args = getopt.getopt(sys.argv[1:], "f:n:h")
    pdbfile='-1'
    chain_number = 1  ##default the first chain
    for o, a in opts:
        if o == "-f":
            pdbfile = a
        elif o == "-n":
            chain_number=int(a)
        elif o == '-h':
            usage()
            sys.exit(0)

    if (pdbfile == '-1'):
        print('\n This script visualize the distance map from the input of a PDB structure.\n')
        usage()
        exit(1)



    pdb=parse_pdb(pdbfile,chain_number)
    xyz=get_xyz(pdb)
    print("computing distance and orientation...")
    output(xyz)
    print("the visualization is in seq.natdist.png")
    print("done")
   
    

if __name__ == "__main__":
    main()

